﻿using System;
using System.Collections.Generic;
using System.Threading.Tasks;
using ExcelDna.Integration;

public static class MyAddInClass
{
    public static int AnsCount(object[,] range, int trg)
    {
        // 組合せ探索
        CombiClass Combi = new CombiClass(trg, range);
        List<int[]> AnsList = Combi.EnumCombi(range);
        return AnsList.Count;
    }

    public static object[,] EnumCombi(object[,] range, int trg)
    {
        // 配列数式範囲参照処理
        ExcelReference caller = (ExcelReference)XlCall.Excel(XlCall.xlfCaller);
        int r_count = caller.RowLast - caller.RowFirst + 1;
        int c_count = caller.ColumnLast - caller.ColumnFirst + 1;
        object[,] result = new object[r_count, c_count];

        // 組合せ探索
        CombiClass Combi = new CombiClass(trg, range);
        List<int[]> AnsList = Combi.EnumCombi(range);

        // 配列サイズチェック
        if (AnsList.Count < r_count)
        {
            r_count = AnsList.Count;
        }

        // 戻り値生成処理
        for (int r = 0; r < r_count; r++)
        {
            for (int c = 0; c < c_count; c++)
            {
                result[r, c] = AnsList[r][c];
            }
        }
        return result;
    }
}


public class CombiClass
{
    public List<int[]> AnsList;
    private readonly int target;   // 目標値
    private readonly int[,] table; // 探索対象配列
    private readonly int r_count;  // 配列の行数
    private readonly int c_count;  // 配列の列数

    // コンストラクタ
    public CombiClass(int trg, object[,] range)
    {
        // クラス変数を設定する
        this.AnsList = new List<int[]>();
        this.target = trg;
        this.r_count = range.GetLength(0);
        this.c_count = range.GetLength(1);
        this.table = cast(range);
    }

    // 列挙メソッド
    public List<int[]> EnumCombi(object[,] range)
    {
        Parallel.For(0, r_count, r =>
            {
                int[] ans = new int[this.c_count]; // 暫定解配列
                ans[0] = table[r, 0];
                int sub_total = table[r, 0];     // 暫定化の合計値
                seek(1, ref ans, sub_total);     // 再帰関数実行
            });
        return this.AnsList;
    }

    // 再帰関数
    private void seek(int col, ref int[] ans, int sub_total)
    {
        if (this.c_count < col + 1) return;
        for (int r = 0; r < r_count; r++)
        {
            if (sub_total + this.table[r, col] > this.target) return;
            int[] tmp = new int[this.c_count];
            ans.CopyTo(tmp, 0);
            tmp[col] = this.table[r, col];
            if (sub_total + this.table[r, col] == this.target) AnsList.Add(tmp); // 解を記録
            if (sub_total + this.table[r, col] < this.target)
            {
                seek(col + 1, ref tmp, sub_total + this.table[r, col]); // 再帰呼び出し
            }
        }
    }

    //型変換
    private int[,] cast(object[,] range)
    {
        int[,] arr = new int[this.r_count, this.c_count];
        for (int r = 0; r < this.r_count; r++)
        {
            for (int c = 0; c < this.c_count; c++)
            {
                arr[r, c] = Convert.ToInt32(range[r, c]);
            }
        }
        return arr;
    }

}